// ComplexInterpoly.cpp:      .
//

#include "stdafx.h"

#define FloatType double
#define PI 3.1415926535897932384626433832795

struct COMPLEX
{
	FloatType Re;
	FloatType Im;
};

struct COMPLEX_POINT
{
	COMPLEX x;
	COMPLEX y;
};

void AddComplex(COMPLEX* Dst, COMPLEX* a, COMPLEX* b)
{
	Dst->Re = a->Re + b->Re;
	Dst->Im = a->Im + b->Im;
}

void SubComplex(COMPLEX* Dst, COMPLEX* Minuent, COMPLEX* Subtrahend)
{
	Dst->Re = Minuent->Re - Subtrahend->Re;
	Dst->Im = Minuent->Im - Subtrahend->Im;
}

void MultComplex(COMPLEX* Dst, COMPLEX* a, COMPLEX* b)
{
	FloatType Re;
	
	Re = a->Re*b->Re - a->Im*b->Im;
	Dst->Im = a->Re*b->Im + a->Im*b->Re;
	Dst->Re = Re;
}

void DivComplex(COMPLEX* Dst, COMPLEX* Num, COMPLEX* Denom)
{
	COMPLEX s;
	s.Re = Denom->Re;
	s.Im = -Denom->Im;

	FloatType d = Denom->Re*Denom->Re + Denom->Im*Denom->Im;

	MultComplex(Dst, Num, &s);
	Dst->Re /= d;
	Dst->Im /= d;
}

void IntPowComplex(COMPLEX* Dst, COMPLEX* Base, int Exponent)
{
	Dst->Re = 1;
	Dst->Im = 0;
	if (Exponent != 0)
	{
		for (unsigned int i = 0; i < abs(Exponent); i++)
		{
			MultComplex(Dst, Dst, Base);
		}

		if (Exponent < 0)
		{
			COMPLEX tmp;
			tmp.Re = 1;
			tmp.Im = 0;

			DivComplex(Dst, &tmp, Dst);
		}
	}
}

void CalcPoly(COMPLEX Poly[], unsigned int PolyLength, COMPLEX* z, COMPLEX* Result)
{
	double ResRe = 0;
	double ResIm = 0;

	for (unsigned int i = PolyLength; i > 0; i--)
	{
		double tmpRe = z->Re*ResRe - z->Im*ResIm;
		ResIm = z->Re*ResIm + z->Im*ResRe;
		ResRe = tmpRe;

		ResRe += Poly[i - 1].Re;
		ResIm += Poly[i - 1].Im;
	}

	Result->Re = ResRe;
	Result->Im = ResIm;
}

void GetM(COMPLEX_POINT Points[], unsigned int StartIndex, unsigned int MiddleIndex, unsigned int EndIndex, COMPLEX* M)
{
	COMPLEX tmp;

	M->Re = 1;
	M->Im = 0;

	if (MiddleIndex > StartIndex)
	{
		for (unsigned int i = StartIndex; i < MiddleIndex; i++)
		{
			SubComplex(&tmp, &Points[i].x, &Points[MiddleIndex].x);
			MultComplex(M, M, &tmp);
		}
	}

	if (MiddleIndex < EndIndex)
	{
		for (unsigned int i = MiddleIndex + 1; i <= EndIndex; i++)
		{
			SubComplex(&tmp, &Points[MiddleIndex].x, &Points[i].x);
			MultComplex(M, M, &tmp);
		}
	}
}

void GetCoef(COMPLEX_POINT Points[], unsigned int NumOfPoints, COMPLEX LocArr[], unsigned int index, COMPLEX* Result)
{
	Result->Re = 0;
	Result->Im = 0;

	COMPLEX* tmp = (COMPLEX*)malloc(sizeof(COMPLEX)*(index + 1));

	for (unsigned int i = 0; i <= index; i++)
	{
		COMPLEX M;
		GetM(Points, 0, i, index, &M);

		if (i % 2 == 1)
		{
			tmp[i].Re = -LocArr[i].Re;
			tmp[i].Im = -LocArr[i].Im;
		}
		else
		{
			tmp[i].Re = LocArr[i].Re;
			tmp[i].Im = LocArr[i].Im;
		}
		DivComplex(&tmp[i], &tmp[i], &M);
	}

	//sort
	bool Sorted;
	do
	{
		Sorted = true;

		for (unsigned int i = 0; i < index; i++)
		{
			if (tmp[i].Re>tmp[i + 1].Re)
			{
				Sorted = false;

				FloatType g = tmp[i].Re;
				tmp[i].Re = tmp[i + 1].Re;
				tmp[i + 1].Re = g;
			}
		}
	} while (!Sorted);

	do
	{
		Sorted = true;

		for (unsigned int i = 0; i < index; i++)
		{
			if (tmp[i].Im>tmp[i + 1].Im)
			{
				Sorted = false;

				FloatType g = tmp[i].Im;
				tmp[i].Im = tmp[i + 1].Im;
				tmp[i + 1].Im = g;
			}
		}
	} while (!Sorted);
	//

	for (unsigned int i = 0; i <= index; i++)
	{
		AddComplex(Result, Result, &tmp[i]);
	}

	free(tmp);
}

void Interpoly(COMPLEX_POINT Points[], unsigned int NumOfPoints, COMPLEX Poly[])
{
	if (NumOfPoints > 0)
	{
		COMPLEX* LocArr = (COMPLEX*)malloc(sizeof(COMPLEX)*NumOfPoints);
		
		for (unsigned int i = 0; i < NumOfPoints; i++)
		{
			LocArr[i] = Points[i].y;
		}

		for (unsigned int i = NumOfPoints; i>0; i--)
		{
			GetCoef(Points, NumOfPoints, LocArr, i - 1, &Poly[i - 1]);

			for (unsigned int j = 0; j < i; j++)
			{
				COMPLEX tmp;

				IntPowComplex(&tmp, &Points[j].x, i - 1);
				MultComplex(&tmp, &tmp, &Poly[i - 1]);
				SubComplex(&LocArr[j], &LocArr[j], &tmp);
			}
		}

		free(LocArr);
	}
}

#define Threshold 2

FloatType CalcErrOnNPoints(unsigned int N)
{
	COMPLEX_POINT* Points = (COMPLEX_POINT*)malloc(N*sizeof(COMPLEX_POINT));
	COMPLEX* Poly = (COMPLEX*)malloc(N*sizeof(COMPLEX));

	for (unsigned int i = 0; i < N; i++)
	{
		Points[i].x.Re = (2 * Threshold*(FloatType)rand() / (FloatType)RAND_MAX) - Threshold;
		Points[i].x.Im = (2 * Threshold*(FloatType)rand() / (FloatType)RAND_MAX) - Threshold;

		Points[i].y.Re = (2 * Threshold*(FloatType)rand() / (FloatType)RAND_MAX) - Threshold;
		Points[i].y.Im = (2 * Threshold*(FloatType)rand() / (FloatType)RAND_MAX) - Threshold;
	}

	Interpoly(Points, N, Poly);

	FloatType TotErr = 0;
	for (unsigned int i = 0; i < N; i++)
	{
		COMPLEX Val;
		CalcPoly(Poly, N, &Points[i].x, &Val);
		SubComplex(&Val, &Points[i].y, &Val);

		FloatType d = sqrt(Val.Re*Val.Re + Val.Im*Val.Im);
		TotErr += d;
	}
	TotErr /= N;

	free(Poly);
	free(Points);

	return TotErr;
}

FloatType CalcErrOnMItersNPoints(unsigned int M, unsigned int N)
{
	FloatType Result = 0;
	for (unsigned int i = 0; i < M; i++)
	{
		Result += CalcErrOnNPoints(N);
	}
	Result /= M;

	return Result;
}

#define ArrSize 100

int _tmain(int argc, _TCHAR* argv[])
{
	LARGE_INTEGER q;
	QueryPerformanceCounter(&q);
	srand(q.HighPart^q.LowPart);

	/*FloatType Err = CalcErrOnMItersNPoints(500, ArrSize);
	printf("\nErr=%e\n", Err);*/

	FILE* f;
	fopen_s(&f, "C:\\temp\\Interpoly.csv", "w");

	fwrite("Poly deg;Error\n", strlen("Poly deg;Error\n"), 1, f);

	unsigned int Pc, PrevPc = 0;

	for (unsigned int i = 1; i <= ArrSize; i++)
	{
		FloatType Err = CalcErrOnMItersNPoints(1000, i);

		char s[256];
		sprintf_s(s, 256, "%u;%e\n", i, Err);

		fwrite(s, strlen(s), 1, f);

		Pc = 100 * i / ArrSize;
		if (Pc != PrevPc)
		{
			PrevPc = Pc;
			printf("%u%% - done\n", Pc);
		}
	}

	fclose(f);
	printf("DONE!\n");

	int zxc;
	scanf_s("%d", &zxc);
	return 0;
}

