#include "stdafx.h"
#include "ComplexN.h"

void ComplexInit(COMPLEX* Dst, FloatType Re, FloatType Im)
{
	FloatType aa = sqrt(Re*Re + Im*Im);
	if (aa > 0)
	{
		Dst->A = aa;
		Dst->NRe = Re / Dst->A;
		Dst->NIm = Im / Dst->A;
	}
	else
	{
		Dst->A = 0;
		Dst->NRe = 0;
		Dst->NIm = 0;
	}
}

void ComplexRandom(COMPLEX* Dst, FloatType MinRe, FloatType MaxRe, FloatType MinIm, FloatType MaxIm)
{
	FloatType Re = ((MaxRe - MinRe)*(FloatType)rand() / (FloatType)RAND_MAX) + MinRe;
	FloatType Im = ((MaxIm - MinIm)*(FloatType)rand() / (FloatType)RAND_MAX) + MinIm;
	ComplexInit(Dst, Re, Im);
}

FloatType ComplexRe(COMPLEX* z)
{
	return z->A*z->NRe;
}

FloatType ComplexIm(COMPLEX* z)
{
	return z->A*z->NIm;
}

void ComplexAdd(COMPLEX* Dst, COMPLEX* A, COMPLEX* B)
{
	if (A->A <= 0)
	{
		*Dst = *B;
		return;
	}
	if (B->A <= 0)
	{
		*Dst = *A;
		return;
	}

	FloatType Re = A->NRe*A->A + B->NRe*B->A;
	FloatType Im = A->NIm*A->A + B->NIm*B->A;

	FloatType aa = sqrt(Re*Re + Im*Im);
	if (aa <= 0)
	{
		Dst->A = 0;
		Dst->NRe = 0;
		Dst->NIm = 0;
		return;
	}

	Dst->A = aa;
	Dst->NRe = Re / aa;
	Dst->NIm = Im / aa;
}

void ComplexSub(COMPLEX* Dst, COMPLEX* Minuent, COMPLEX* Subtrahend)
{
	if (Minuent->A <= 0)
	{
		*Dst = *Subtrahend;
		return;
	}
	if (Subtrahend->A <= 0)
	{
		*Dst = *Minuent;
		return;
	}

	FloatType Re = Minuent->NRe*Minuent->A - Subtrahend->NRe*Subtrahend->A;
	FloatType Im = Minuent->NIm*Minuent->A - Subtrahend->NIm*Subtrahend->A;

	FloatType aa = sqrt(Re*Re + Im*Im);
	if (aa <= 0)
	{
		Dst->A = 0;
		Dst->NRe = 0;
		Dst->NIm = 0;
		return;
	}

	Dst->A = aa;
	Dst->NRe = Re / aa;
	Dst->NIm = Im / aa;
}

void ComplexMul(COMPLEX* Dst, COMPLEX* A, COMPLEX* B)
{
	Dst->A = A->A*B->A;
	FloatType Re = A->NRe*B->NRe - A->NIm*B->NIm;
	FloatType Im = A->NRe*B->NIm + A->NIm*B->NRe;
	//  Re  Im !!!
	/*FloatType a = sqrt(Re*Re + Im*Im);
	Re /= a;
	Im /= a;*/

	Dst->NRe = Re;
	Dst->NIm = Im;
}

void ComplexDiv(COMPLEX* Dst, COMPLEX* Num, COMPLEX* Denom)
{
	COMPLEX tmp;
	tmp.A = 1;
	tmp.NRe = Denom->NRe;
	tmp.NIm = -Denom->NIm;

	ComplexMul(Dst, Num, &tmp);

	Dst->A /= Denom->A;
}

void ComplexIntPow(COMPLEX* Dst, COMPLEX* Base, unsigned int Exponent)
{
	Dst->A = 1;
	Dst->NRe = 1;
	Dst->NIm = 0;

	for (unsigned int i = 0; i < Exponent; i++)
	{
		ComplexMul(Dst, Dst, Base);
	}
}